/*
 * Copyright (c) 2000-2001 Sendmail, Inc. and its suppliers.
 *	All rights reserved.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found at the top level of
 * the sendmail distribution.
 *
 *	$Id: safe.h,v 1.4 2001/04/03 01:53:00 gshapiro Exp $
 */

/*
**  libsm file security checking.
*/

#ifndef SM_SAFEFILE_H
# define SM_SAFEFILE_H

# include <sm/gen.h>
# include <sm/exc.h>
# include <sys/stat.h>

/*
**  flags member of SM_SAFE_PARAMS_T
*/

#define SM_SAFE_ANYFILE	     0L		/* no special restrictions */
#define SM_SAFE_MUSTOWN	     0x0000001L	/* user must own this file */
#define SM_SAFE_NOSLINK	     0x0000002L	/* file cannot be a symbolic link */
#define SM_SAFE_ROOTOK	     0x0000004L	/* ok for root to own this file */
#define SM_SAFE_RUNASREALUID 0x0000008L	/* if no ctladdr, run as real uid */
#define SM_SAFE_NOPATHCHECK  0x0000010L	/* don't bother checking dir path */
#define SM_SAFE_SETUIDOK     0x0000020L	/* setuid files are ok */
#define SM_SAFE_CREAT	     0x0000040L	/* ok to create file if necessary */
#define SM_SAFE_REGONLY	     0x0000080L	/* regular files only */
#define SM_SAFE_NOWDIRS      0x0000100L	/* disallow world/group writable dirs */
#define SM_SAFE_NOHLINK	     0x0000200L	/* file cannot have hard links */
#define SM_SAFE_NOWLINK	     0x0000400L	/* links only in non-writable dirs */
#define SM_SAFE_NOGWFILES    0x0000800L	/* disallow world writable files */
#define SM_SAFE_NOWWFILES    0x0001000L	/* disallow group writable files */
#define SM_SAFE_OPENASROOT   0x0002000L	/* open as root instead of real user */
#define SM_SAFE_NOLOCK	     0x0004000L	/* don't lock the file */
#define SM_SAFE_NOGRFILES    0x0008000L	/* disallow g readable files */
#define SM_SAFE_NOWRFILES    0x0010000L	/* disallow o readable files */
#define SM_SAFE_NOTEXCL	     0x0020000L	/* creates are not exclusive */
#define SM_SAFE_EXECOK	     0x0040000L	/* executable files are ok */
#define SM_SAFE_SUIDROOTOK   0x0080000L /* setuid root files are ok */
#define SM_SAFE_TRUSTVTX     0x0100000L /* trust directory sticky bit */
#define SM_SAFE_GWDIROK	     0x0200000L /* allow group writable directories */
#define SM_SAFE_NOGROUPS     0x0400000L /* ignore group list */

#define SM_SAFE_NORFILES     (SM_SAFE_NOGRFILES|SM_SAFE_NOWRFILES)
#define SM_SAFE_NOLINK	     (SM_SAFE_NOHLINK|SM_SAFE_NOSLINK)
#define SM_SAFE_SAFEDIRPATH  SM_SAFE_NOWDIRS

typedef struct {
	uid_t	uid;
	gid_t	gid;
	char	*user;
	uid_t	trusteduid;
	long	flags;
	int	mode;
	void	(*warning) __P((SM_EXC_T *_exc, void *_wcontext));
	void	*wcontext;
} SM_SAFE_PARAMS_T;

void
sm_safe_file_x __P((
	char *_filename,
	SM_SAFE_PARAMS_T *_params,
	struct stat *_st));

void
sm_safe_dirpath_x __P((
	char *_filename,
	SM_SAFE_PARAMS_T *_params,
	int _level,
	int _offset));

#if 0
int
sm_safeopen __P((
	char *_fn,
	int _omode,
	int _cmode,
	long _sff));

FILE *
sm_safefopen __P((
	char *_fn,
	int _omode,
	int _cmode,
	long _sff));

bool
sm_filechanged __P((
	char *_fn,
	int _fd,
	struct stat *_stb));

int
sm_dfopen __P((
	char *_filename,
	int _omode,
	int _cmode,
	long _sff));
#endif

#endif /* SM_SAFEFILE_H */
